==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/fir/analysis/native/checkers/FirNativeSharedImmutableChecker.kt <==
/*
 * Copyright 2010-2022 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.analysis.native.checkers

import ksp.org.jetbrains.kotlin.KtFakeSourceElementKind
import ksp.org.jetbrains.kotlin.diagnostics.DiagnosticReporter
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.MppCheckerKind
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.context.CheckerContext
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.declaration.FirBasicDeclarationChecker
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.isTopLevel
import ksp.org.jetbrains.kotlin.fir.analysis.diagnostics.native.FirNativeErrors
import ksp.org.jetbrains.kotlin.fir.declarations.FirDeclaration
import ksp.org.jetbrains.kotlin.fir.declarations.FirProperty
import ksp.org.jetbrains.kotlin.fir.declarations.FirValueParameter
import ksp.org.jetbrains.kotlin.fir.declarations.FirVariable
import ksp.org.jetbrains.kotlin.fir.declarations.impl.FirPrimaryConstructor
import ksp.org.jetbrains.kotlin.fir.declarations.utils.hasBackingField
import ksp.org.jetbrains.kotlin.name.ClassId
import ksp.org.jetbrains.kotlin.name.FqName

object FirNativeSharedImmutableChecker : FirBasicDeclarationChecker(MppCheckerKind.Platform) {
    private val sharedImmutableClassId = ClassId.topLevel(FqName("kotlin.native.concurrent.SharedImmutable"))

    override fun check(declaration: FirDeclaration, context: CheckerContext, reporter: DiagnosticReporter) {
        if (declaration is FirVariable) {
            if (declaration !is FirValueParameter || context.containingDeclarations.lastOrNull() !is FirPrimaryConstructor) {
                val hasBackingField = declaration is FirProperty && declaration.hasBackingField
                if ((declaration.isVar || !hasBackingField) && declaration.delegate == null) {
                    reporter.reportIfHasAnnotation(
                        declaration,
                        sharedImmutableClassId,
                        FirNativeErrors.INAPPLICABLE_SHARED_IMMUTABLE_PROPERTY,
                        context
                    )
                }
            }
        } else {
            if (declaration.source?.kind is KtFakeSourceElementKind) return

            reporter.reportIfHasAnnotation(
                declaration,
                sharedImmutableClassId,
                FirNativeErrors.INAPPLICABLE_SHARED_IMMUTABLE_PROPERTY,
                context
            )

            return
        }

        if (!context.isTopLevel) {
            reporter.reportIfHasAnnotation(
                declaration,
                sharedImmutableClassId,
                FirNativeErrors.INAPPLICABLE_SHARED_IMMUTABLE_TOP_LEVEL,
                context
            )
        }
    }
}

==> symbol-processing-aa-embeddable-2.2.0-2.0.2-sources.jar: org/jetbrains/kotlin/fir/analysis/native/checkers/FirNativeSharedImmutableChecker.kt <==
/*
 * Copyright 2010-2022 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.fir.analysis.native.checkers

import ksp.org.jetbrains.kotlin.KtFakeSourceElementKind
import ksp.org.jetbrains.kotlin.diagnostics.DiagnosticReporter
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.MppCheckerKind
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.context.CheckerContext
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.declaration.FirBasicDeclarationChecker
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.isPrimaryConstructor
import ksp.org.jetbrains.kotlin.fir.analysis.checkers.isTopLevel
import ksp.org.jetbrains.kotlin.fir.analysis.diagnostics.native.FirNativeErrors
import ksp.org.jetbrains.kotlin.fir.declarations.FirDeclaration
import ksp.org.jetbrains.kotlin.fir.declarations.FirProperty
import ksp.org.jetbrains.kotlin.fir.declarations.FirValueParameter
import ksp.org.jetbrains.kotlin.fir.declarations.FirVariable
import ksp.org.jetbrains.kotlin.fir.declarations.utils.hasBackingField
import ksp.org.jetbrains.kotlin.name.ClassId
import ksp.org.jetbrains.kotlin.name.FqName

object FirNativeSharedImmutableChecker : FirBasicDeclarationChecker(MppCheckerKind.Platform) {
    private val sharedImmutableClassId = ClassId.topLevel(FqName("kotlin.native.concurrent.SharedImmutable"))

    context(context: CheckerContext, reporter: DiagnosticReporter)
    override fun check(declaration: FirDeclaration) {
        if (declaration is FirVariable) {
            if (declaration !is FirValueParameter || context.containingDeclarations.lastOrNull().isPrimaryConstructor()) {
                val hasBackingField = declaration is FirProperty && declaration.hasBackingField
                if ((declaration.isVar || !hasBackingField) && declaration.delegate == null) {
                    reporter.reportIfHasAnnotation(
                        declaration,
                        sharedImmutableClassId,
                        FirNativeErrors.INAPPLICABLE_SHARED_IMMUTABLE_PROPERTY,
                        context
                    )
                }
            }
        } else {
            if (declaration.source?.kind is KtFakeSourceElementKind) return

            reporter.reportIfHasAnnotation(
                declaration,
                sharedImmutableClassId,
                FirNativeErrors.INAPPLICABLE_SHARED_IMMUTABLE_PROPERTY,
                context
            )

            return
        }

        if (!context.isTopLevel) {
            reporter.reportIfHasAnnotation(
                declaration,
                sharedImmutableClassId,
                FirNativeErrors.INAPPLICABLE_SHARED_IMMUTABLE_TOP_LEVEL,
                context
            )
        }
    }
}
